/*
 * GatesGUI.java
 *
 * Created on May 19, 2007, 11:29 PM
 */

package gui;
import nn.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * Neural Netoworks Simulator GUI
 * @author  Tomasz Gebarowski
 */
public class GatesGUI extends javax.swing.JFrame implements ChangeListener {
    
    // Neural Network Configuration
    private int nInputs;
    private int nOutputs;
    private int nHiddens;
    
    // Number of iterations to teach network
    private int totalSteps;
    
    // Reference to Neural Network
    private BackPropagationNN nn;
    
    /** Creates new form GatesGUI */
    public GatesGUI() {
        initComponents();

        // Set Progress Bar properties
        progressBar.setMinimum(0);
        progressBar.setMaximum(100);
        progressBar.setStringPainted(true);
        
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        tabbedPanel = new javax.swing.JTabbedPane();
        settingsPanel = new javax.swing.JPanel();
        hiddenLabel = new javax.swing.JLabel();
        labelInputs = new javax.swing.JLabel();
        jLabel1 = new javax.swing.JLabel();
        hiddenNum = new javax.swing.JComboBox();
        inputsNum = new javax.swing.JComboBox();
        stepsLabel = new javax.swing.JLabel();
        inputSteps = new javax.swing.JComboBox();
        outputsNum = new javax.swing.JComboBox();
        trainerPanel = new javax.swing.JPanel();
        captionLabel = new javax.swing.JLabel();
        trainDataTable = new javax.swing.JScrollPane();
        jTable1 = new javax.swing.JTable();
        trainButton = new javax.swing.JButton();
        resultLabel = new javax.swing.JLabel();
        progressBar = new javax.swing.JProgressBar();
        predefinedInput = new javax.swing.JComboBox();
        testerData = new javax.swing.JPanel();
        jScrollPane3 = new javax.swing.JScrollPane();
        testDataTable = new javax.swing.JTable();
        testButton = new javax.swing.JButton();
        inputsLabel = new javax.swing.JLabel();
        testResultLabel = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        resultTable = new javax.swing.JTable();
        aboutPanel = new javax.swing.JPanel();
        appTitleLabel = new javax.swing.JLabel();
        authorsLabel = new javax.swing.JLabel();
        a1Label = new javax.swing.JLabel();
        rightsLabel = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setName("Artificial Neural Networks"); // NOI18N

        hiddenLabel.setText("Hidden Layers");

        labelInputs.setText("Inputs");

        jLabel1.setText("Outputs");

        hiddenNum.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "5", "4", "3", "2", "1", "6", "7", "8", "9", "10" }));

        inputsNum.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "2", "1", "3", "4" }));
        inputsNum.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                inputsNumItemStateChanged(evt);
            }
        });

        stepsLabel.setText("Number of training steps");

        inputSteps.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "10000", "7500", "5000", "2500", "1000", "100", "50000", "100000" }));

        outputsNum.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "1", "2", "3", "4" }));
        outputsNum.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                outputsNumItemStateChanged(evt);
            }
        });

        org.jdesktop.layout.GroupLayout settingsPanelLayout = new org.jdesktop.layout.GroupLayout(settingsPanel);
        settingsPanel.setLayout(settingsPanelLayout);
        settingsPanelLayout.setHorizontalGroup(
            settingsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(settingsPanelLayout.createSequentialGroup()
                .add(59, 59, 59)
                .add(settingsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(settingsPanelLayout.createSequentialGroup()
                        .add(stepsLabel)
                        .add(15, 15, 15)
                        .add(inputSteps, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 113, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(settingsPanelLayout.createSequentialGroup()
                        .add(settingsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(hiddenLabel)
                            .add(labelInputs)
                            .add(jLabel1))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(settingsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(outputsNum, 0, 192, Short.MAX_VALUE)
                            .add(org.jdesktop.layout.GroupLayout.TRAILING, hiddenNum, 0, 192, Short.MAX_VALUE)
                            .add(org.jdesktop.layout.GroupLayout.TRAILING, inputsNum, 0, 192, Short.MAX_VALUE))))
                .add(726, 726, 726))
        );
        settingsPanelLayout.setVerticalGroup(
            settingsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(settingsPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(settingsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(hiddenLabel)
                    .add(hiddenNum, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(14, 14, 14)
                .add(settingsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(labelInputs)
                    .add(inputsNum, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(25, 25, 25)
                .add(settingsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel1)
                    .add(outputsNum, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(36, 36, 36)
                .add(settingsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(inputSteps, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(stepsLabel))
                .addContainerGap(81, Short.MAX_VALUE))
        );

        tabbedPanel.addTab("Neural Network Settings", settingsPanel);

        captionLabel.setText("Truth Table");

        jTable1.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null},
                {null, null, null},
                {null, null, null},
                {null, null, null}
            },
            new String [] {
                "X1", "X2", "Y"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.Boolean.class, java.lang.Boolean.class, java.lang.Boolean.class
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        trainDataTable.setViewportView(jTable1);

        trainButton.setText("Train->");
        trainButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                trainButtonMouseClicked(evt);
            }
        });

        resultLabel.setText("Predefined Values");

        predefinedInput.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "2 Input XOR", "2 Input AND", "2 Input NOR", "2 Input NAND", "2 Input OR", "3 Input AND", "4 Input NAND", "1 Input NOT" }));
        predefinedInput.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                predefinedInputItemStateChanged(evt);
            }
        });

        org.jdesktop.layout.GroupLayout trainerPanelLayout = new org.jdesktop.layout.GroupLayout(trainerPanel);
        trainerPanel.setLayout(trainerPanelLayout);
        trainerPanelLayout.setHorizontalGroup(
            trainerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, trainerPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(trainerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(captionLabel)
                    .add(trainerPanelLayout.createSequentialGroup()
                        .add(1, 1, 1)
                        .add(trainDataTable, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 271, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .add(22, 22, 22)
                .add(trainerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(predefinedInput, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 208, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(resultLabel)
                    .add(trainButton))
                .add(94, 94, 94))
            .add(trainerPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(progressBar, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 582, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        trainerPanelLayout.setVerticalGroup(
            trainerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(trainerPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(trainerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(captionLabel)
                    .add(resultLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(trainerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(trainerPanelLayout.createSequentialGroup()
                        .add(predefinedInput, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(16, 16, 16)
                        .add(trainButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 49, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(trainDataTable, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 188, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(progressBar, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        tabbedPanel.addTab("Network Trainer", trainerPanel);

        testDataTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null}
            },
            new String [] {
                "X1", "X2"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.Boolean.class, java.lang.Boolean.class
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        jScrollPane3.setViewportView(testDataTable);

        testButton.setText("Test->");
        testButton.setEnabled(false);
        testButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                testButtonActionPerformed(evt);
            }
        });

        inputsLabel.setText("Inputs");

        testResultLabel.setText("Result");

        resultTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null}
            },
            new String [] {
                "Y0"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.Boolean.class
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        jScrollPane1.setViewportView(resultTable);

        org.jdesktop.layout.GroupLayout testerDataLayout = new org.jdesktop.layout.GroupLayout(testerData);
        testerData.setLayout(testerDataLayout);
        testerDataLayout.setHorizontalGroup(
            testerDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(testerDataLayout.createSequentialGroup()
                .add(testerDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(testerDataLayout.createSequentialGroup()
                        .add(44, 44, 44)
                        .add(inputsLabel))
                    .add(testerDataLayout.createSequentialGroup()
                        .addContainerGap()
                        .add(jScrollPane3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 143, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .add(28, 28, 28)
                .add(testButton)
                .add(34, 34, 34)
                .add(testerDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(testResultLabel)
                    .add(jScrollPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 77, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(211, Short.MAX_VALUE))
        );
        testerDataLayout.setVerticalGroup(
            testerDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(testerDataLayout.createSequentialGroup()
                .add(16, 16, 16)
                .add(testerDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(testButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 52, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(testerDataLayout.createSequentialGroup()
                        .add(testerDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(inputsLabel)
                            .add(testResultLabel))
                        .add(20, 20, 20)
                        .add(testerDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(jScrollPane3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 42, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jScrollPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 40, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))
                .addContainerGap(186, Short.MAX_VALUE))
        );

        tabbedPanel.addTab("Network Tester", testerData);

        appTitleLabel.setFont(new java.awt.Font("Lucida Grande", 0, 18));
        appTitleLabel.setText("Artificial Neural Networks Logic Gates Simulator");

        authorsLabel.setText("Author:");

        a1Label.setText("Tomasz Gebarowski");

        rightsLabel.setFont(new java.awt.Font("Lucida Grande", 0, 12));
        rightsLabel.setText("Program realised for Robotics Project at Windesheim University of Applied Sciences, Zwolle 2007.");

        org.jdesktop.layout.GroupLayout aboutPanelLayout = new org.jdesktop.layout.GroupLayout(aboutPanel);
        aboutPanel.setLayout(aboutPanelLayout);
        aboutPanelLayout.setHorizontalGroup(
            aboutPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(aboutPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(aboutPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(appTitleLabel)
                    .add(aboutPanelLayout.createSequentialGroup()
                        .add(authorsLabel)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                        .add(a1Label))
                    .add(rightsLabel))
                .addContainerGap(15, Short.MAX_VALUE))
        );
        aboutPanelLayout.setVerticalGroup(
            aboutPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(aboutPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(appTitleLabel)
                .add(29, 29, 29)
                .add(aboutPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(authorsLabel)
                    .add(a1Label))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 168, Short.MAX_VALUE)
                .add(rightsLabel)
                .addContainerGap())
        );

        rightsLabel.getAccessibleContext().setAccessibleName("Program realised for Robotics Project at Windesheim University of Applied Sciences.");

        tabbedPanel.addTab("About", aboutPanel);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(tabbedPanel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 619, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(tabbedPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 326, Short.MAX_VALUE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /* Handle Test Button */
    private void testButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_testButtonActionPerformed
       double in[] = new double[nInputs]; 
       double out[] = new double[nOutputs]; 
       
       javax.swing.table.TableModel m = this.testDataTable.getModel();
   
       for ( int i = 0; i < nInputs; i++ ) {
          in[i] = doubleFromBoolean( Boolean.valueOf( String.valueOf( m.getValueAt(0,i) ) ) );
       }
       
       nn.processForward(in);
       
       javax.swing.table.TableModel m2 = this.resultTable.getModel();
       for ( int i = 0; i < nOutputs; i++ ) {
           m2.setValueAt( Boolean.valueOf(  booleanFromDouble( nn.outputs[i] ) ), 0, i  );           
           System.out.println( nn.outputs[i] );
       }
       this.resultTable.setModel(m2);
    }//GEN-LAST:event_testButtonActionPerformed

    /* Handle Predefined items Changed ComboBox */
    private void predefinedInputItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_predefinedInputItemStateChanged

        String selectedItem = (String)evt.getItem();
        
        if ( selectedItem.equals("2 Input XOR") ) {
            Boolean[][] truthTable = new Boolean[][] { 
                { false, false, false },
                { false, true, true },
                { true, false, true },
                { true, true, false }                
            }; 
            nInputs = 2;
            inputsNum.setSelectedIndex(0);
            jTable1.setModel(createModel(  nInputs ,1, truthTable ) );         
        } else if ( selectedItem.equals("2 Input AND") ) {
            Boolean[][] truthTable = new Boolean[][] { 
                { false, false, false },
                { false, true, false },
                { true, false, false },
                { true, true, true }                
            };   
            nInputs = 2;
            inputsNum.setSelectedIndex(0);
            jTable1.setModel(createModel(  nInputs ,1, truthTable ) );     
        }  else if ( selectedItem.equals("2 Input NAND") ) {
            Boolean[][] truthTable = new Boolean[][] { 
                { false, false, true },
                { false, true, true },
                { true, false, true },
                { true, true, false }                
            };   
            nInputs = 2;
            inputsNum.setSelectedIndex(0);
            jTable1.setModel(createModel(  nInputs ,1, truthTable ) );     
        }  else if ( selectedItem.equals("2 Input OR") ) {
            Boolean[][] truthTable = new Boolean[][] { 
                { false, false, false },
                { false, true, true },
                { true, false, true },
                { true, true, true }                
            };     
            nInputs = 2;
            inputsNum.setSelectedIndex(0);
            jTable1.setModel(createModel(  nInputs ,1, truthTable ) );    
        } else if ( selectedItem.equals("2 Input NOR") ) {
            Boolean[][] truthTable = new Boolean[][] { 
                { false, false, true },
                { false, true, false },
                { true, false, false },
                { true, true, false }                
            };             
            nInputs = 2;
            inputsNum.setSelectedIndex(0);
            jTable1.setModel(createModel(  nInputs ,1, truthTable ) );     
        } else if ( selectedItem.equals("3 Input AND") ) {
            Boolean[][] truthTable = new Boolean[][] { 
                { false, false, false, false  },
                { false, false, true, false  },
                { false, true, false, false  },
                { false, true, true, false  },
                { true, false, false, false  },
                { true, false, true, false  },
                { true, true, false, false  },
                { true, true, true, true  }             
            };                            
            nInputs = 3;
            inputsNum.setSelectedIndex(2);
            jTable1.setModel(createModel(  nInputs ,1, truthTable ) );
        } else if ( selectedItem.equals("4 Input NAND") ) {
            Boolean[][] truthTable = new Boolean[][] { 
                { false, false, false, false, true  },   
                { false, false, false, true, true  },
                { false, false, true, false, true  },
                { false, false, true, true, true  },
                { false, true, false, false, true  },
                { false, true, false, true, true  },
                { false, true, true, false, true  },
                { false, true, true, true, true  },
                { true, false, false, false, true  },
                { true, false, false, true, true  },
                { true, false, true, false, true  },
                { true, false, true, true, true  },
                { true, true, false, false, true  },
                { true, true, false, true, true  },
                { true, true, true, false, true  },
                { true, true, true, true, false  }
          
            };            
            nInputs = 4;
            inputsNum.setSelectedIndex(3);
            jTable1.setModel(createModel(  nInputs ,1, truthTable ) );     
        } else if ( selectedItem.equals("1 Input NOT") ) {
            Boolean[][] truthTable = new Boolean[][] { 
                { false, true },
                { true, false }                
            };   
            nInputs = 1;
            inputsNum.setSelectedIndex(1);
            jTable1.setModel(createModel(  nInputs ,1, truthTable ) );     
        }  
        
        
        
    }//GEN-LAST:event_predefinedInputItemStateChanged

    /**
     * Creates TableModel for the JTable component basing on number of inputs, outputs and truth table
     * @param nInputs int Number of inputs
     * @param nOutputs int Number of outputs
     * @param truthTable Boolean[][] Truth table content
     * @return DefaultTableModel Created JTable Model
     */ 
    private javax.swing.table.DefaultTableModel createModel( int nInputs, int nOutputs, Boolean[][] truthTable ) {
        
        int totalCols = nInputs+nOutputs;
   
        // Generate Captions
        String captions[] = new String[totalCols];
        
        int lastI = 0;
        
        
        for ( int i = 0; i < nInputs; i++ ) {
            captions[i] = "X" + i;
        }
        
        
        for ( int i = nInputs; i < totalCols; i++ )  {
            int index = i - nInputs;
            captions[i] = "Y" + index ;
        }
        
        // Generate Table Entries Definitions
        final Class[] t = new Class [totalCols];
        for ( int i = 0; i < totalCols; i++ ) {
                t[i] = java.lang.Boolean.class;
        }
      
        javax.swing.table.DefaultTableModel tM = new javax.swing.table.DefaultTableModel(truthTable, captions) {
            Class[] types = t;

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        };
        return tM;
    }
    
    
    /* Handle Output Number Combo Box changed event */
    private void outputsNumItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_outputsNumItemStateChanged
        inputsNumItemStateChanged(evt);
    }//GEN-LAST:event_outputsNumItemStateChanged

    /* Handle Input Number Combo Box changed event */
    private void inputsNumItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_inputsNumItemStateChanged
        
        
                
        nInputs = Integer.valueOf( String.valueOf(inputsNum.getSelectedItem()) ).intValue();
        nHiddens = Integer.valueOf( String.valueOf(hiddenNum.getSelectedItem()) ).intValue();
        nOutputs = Integer.valueOf( String.valueOf(outputsNum.getSelectedItem()) ).intValue();
        
        int totalCols = nInputs+nOutputs;
        
        int inputVar = (int)Math.pow( 2, nInputs);
        Boolean[][] entries = new Boolean [inputVar][totalCols];
        
        for ( int i = 0; i < totalCols; i++ ) {
            for ( int j = 0; j < inputVar; j++ ) {
                entries[j][i] = new Boolean(false);
            }
        }
        
        /* Generate Captions */
        String captions[] = new String[totalCols];
        
        int lastI = 0;
        
        
        for ( int i = 0; i < nInputs; i++ ) {
            captions[i] = "X" + i;
        }
        
        
        for ( int i = nInputs; i < totalCols; i++ )  {
            int index = i - nInputs;
            captions[i] = "Y" + index ;
        }
        
        
        /*Define JTable Content */
        final Class[] t = new Class [totalCols];
        for ( int i = 0; i < totalCols; i++ ) {
                t[i] = java.lang.Boolean.class;
        }
        
        
        /* Set Training Data Structure */
        jTable1.setModel(new javax.swing.table.DefaultTableModel(entries,captions ) {
         
            Class[] types = t;
     
            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }


        });
        
        /* Generate Test Data Captions */
        String cap[] = new String[nInputs];
        
        for ( int i = 0; i < nInputs; i++ ) {
            cap[i] = "X" + i;
        }
        
        Object ent[][] =  new Object[1][nInputs];
        
        for ( int i = 0; i < nInputs; i++ )
            ent[0][i] = new Boolean(false);
        
        
        final Class[] t2 = new Class [nInputs];
        for ( int i = 0; i < nInputs; i++ ) {
                t2[i] = java.lang.Boolean.class;
        }
        
        /* Set Test Data Table Structure */
        testDataTable.setModel(new javax.swing.table.DefaultTableModel(ent,cap ) {
            Class[] types = t2;
            
            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        
        /* Set Result Table Captions */
        String cap2[] = new String[nOutputs];
        
        for ( int i = 0; i < nOutputs; i++ ) {
            cap2[i] = "Y" + i;
        }
        
        Object ent2[][] =  new Object[1][nOutputs];
        
        for ( int i = 0; i < nOutputs; i++ )
            ent2[0][i] = new Boolean(false);
        
        /* Set Result Table Structure */
        final Class[] t3 = new Class [nOutputs];
        for ( int i = 0; i < nOutputs; i++ ) {
                t3[i] = java.lang.Boolean.class;
        }
        
        
        resultTable.setModel(new javax.swing.table.DefaultTableModel(ent2,cap2 ) {
            Class[] types = t3;
            
            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        
        
        
        
    }//GEN-LAST:event_inputsNumItemStateChanged

    /* Handle State Changed Event used for changing progress bar */
    public void stateChanged( ChangeEvent e ) {      
        int progress = (int) ( ( ( 1.0 * ( nn.currentStep + 2 ) / totalSteps ) ) * 100.0 ) ;
        
        progressBar.setValue( progress );
        progressBar.setString( String.valueOf( progress ) + "%" );
        
        if ( progress == 100 )
            testButton.setEnabled(true);
        else
            testButton.setEnabled(false);

    }
    
   /** Helper function for converting Boolean to Double
    * @param value boolean Value to be converted
    * @return double Converted value
    */
    private double doubleFromBoolean(boolean value) {
        if ( value )
            return 1.0;
        
        return 0.0;
    }
    
   /** Helper function for converting Double to Boolean
    * @param value boolean Value to be converted
    * @return double Converted value
    */        
    private boolean booleanFromDouble(double value) {
       if ( value > 0.5 )
           return true;
       
       return false;
    }
    
    /* Handling Train Button Pressed Event */
    private void trainButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_trainButtonMouseClicked

        nInputs = Integer.valueOf( String.valueOf(inputsNum.getSelectedItem()) ).intValue();
        nHiddens = Integer.valueOf( String.valueOf(hiddenNum.getSelectedItem()) ).intValue();
        nOutputs = Integer.valueOf( String.valueOf(outputsNum.getSelectedItem()) ).intValue();
        
        /* Create new Neural Network */
        nn = new BackPropagationNN( nInputs , nHiddens, nOutputs );
        nn.addChangeListener(this);
        
        
        /* Get Training Data handler */
        javax.swing.table.TableModel m = this.jTable1.getModel();
       
       
        /* Create train set */
        TestSet ts = new TestSet(nInputs,nOutputs);
        
        
        /* Fill train sets with all the possible train cases */
       for ( int row = 0; row < (int)Math.pow(2, nInputs ); row++ ) {
            
            double in[] = new double[nInputs]; 
            double out[] = new double[nOutputs]; 
           
            /* Add inputs */
            for ( int i = 0; i < nInputs; i++ ) {
                in[i] = doubleFromBoolean( Boolean.valueOf( String.valueOf( m.getValueAt(row,i) ) ) );
            }    
            
            /* Add outputs */
            for ( int i = nInputs; i < nInputs + nOutputs; i++) {
                out[i - nInputs] = doubleFromBoolean( Boolean.valueOf( String.valueOf( m.getValueAt(row,i) ) ) );                
            }

            ts.addTestSet( in, out );
            
       }
        
        /* Get total number of steps */
        totalSteps = Integer.valueOf( String.valueOf(this.inputSteps.getSelectedItem())).intValue();
        
        /* Perform calculation in new thread */
        nn.addTest( totalSteps, ts );
        Thread t = new Thread(nn);
        t.start();
        
    }//GEN-LAST:event_trainButtonMouseClicked

    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel a1Label;
    private javax.swing.JPanel aboutPanel;
    private javax.swing.JLabel appTitleLabel;
    private javax.swing.JLabel authorsLabel;
    private javax.swing.JLabel captionLabel;
    private javax.swing.JLabel hiddenLabel;
    private javax.swing.JComboBox hiddenNum;
    private javax.swing.JComboBox inputSteps;
    private javax.swing.JLabel inputsLabel;
    private javax.swing.JComboBox inputsNum;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JTable jTable1;
    private javax.swing.JLabel labelInputs;
    private javax.swing.JComboBox outputsNum;
    private javax.swing.JComboBox predefinedInput;
    private javax.swing.JProgressBar progressBar;
    private javax.swing.JLabel resultLabel;
    private javax.swing.JTable resultTable;
    private javax.swing.JLabel rightsLabel;
    private javax.swing.JPanel settingsPanel;
    private javax.swing.JLabel stepsLabel;
    private javax.swing.JTabbedPane tabbedPanel;
    private javax.swing.JButton testButton;
    private javax.swing.JTable testDataTable;
    private javax.swing.JLabel testResultLabel;
    private javax.swing.JPanel testerData;
    private javax.swing.JButton trainButton;
    private javax.swing.JScrollPane trainDataTable;
    private javax.swing.JPanel trainerPanel;
    // End of variables declaration//GEN-END:variables
    
}
